@@ -15,13 +15,26 @@ module Sound.SFML (
     startLoopedSound,
     stopLoopedSound,
 
+    -- * background music
+    playMusic,
+    playMusicLooped,
+    stopMusic,
+    pauseMusic,
+
   ) where
 
 
+import Prelude hiding (mapM_)
+
 import Data.Maybe
 import Data.IORef
+import Data.Foldable (forM_, mapM_)
+import Data.Traversable (forM)
 
-import Control.Monad
+import Control.Monad (when)
+import Control.Concurrent.MVar
+
+import System.IO.Unsafe
 
 import Foreign.Ptr
 
@@ -64,7 +77,7 @@ triggerPolySound :: PolySound -> Maybe Float -> IO ()
 triggerPolySound (PolySound _ _ sounds ref) volume = do
     i <- readIORef ref
     let sound = sounds !! i
-    status <- getStatus sound
+    status <- getSoundStatus sound
     when (status == Stopped) $ do
         writeIORef ref ((i + 1) `mod` length sounds)
         sfSound_SetVolume sound ((fromMaybe 1 volume) * 100)
@@ -94,8 +107,9 @@ freeLoopedSound (LoopedSound ptr) =
     sfSound_Destroy ptr
 
 -- | Starts a looped sound.
-startLoopedSound :: LoopedSound -> IO ()
-startLoopedSound (LoopedSound ptr) =
+startLoopedSound :: Maybe Float -> LoopedSound -> IO ()
+startLoopedSound volume (LoopedSound ptr) = do
+    sfSound_SetVolume ptr ((fromMaybe 1 volume) * 100)
     sfSound_Play ptr
 
 -- | Stops a looped sound.
@@ -106,17 +120,60 @@ stopLoopedSound (LoopedSound ptr) =
 
 -- * Music
 
--- There is always only one music lpaying at a single time.
+-- There is always only one music being played at a single point in time.
 
--- | Loads and plays a music file once.
--- playMusic :: FilePath -> IO ()
--- playMusic = error "playMusic"
+-- | Loads and plays a music file once in a background thread.
+-- Stops other music that is playing.
+-- If the current music is Paused and the given file is the same as the one playing,
+-- the music is continued.
+-- The volume is set again in any case.
+playMusic :: FilePath -> Maybe Float -> IO ()
+playMusic = _playMusic False
 
--- | Loads and plays a music file in a loop.
--- Stops if the program ends or 'stopMusic' is called.
--- playMusicLooped :: FilePath -> IO ()
--- playMusicLooped = error "playMusicLooped"
+-- |Like 'playMusic', but looping.
+playMusicLooped :: FilePath -> Maybe Float -> IO ()
+playMusicLooped = _playMusic True
+
+_playMusic :: Bool -> FilePath -> Maybe Float -> IO ()
+_playMusic looped file volume = modifyMVar_ _globalMusic $ \ mOldMusic -> do
+    case mOldMusic of
+        Just (oldFile, oldMusic) -> do
+            status <- getMusicStatus oldMusic
+            case status of
+                Paused | file == oldFile -> do
+                    sfMusic_SetLoop oldMusic looped
+                    sfMusic_SetVolume oldMusic ((fromMaybe 1 volume) * 100)
+                    sfMusic_Play oldMusic
+                    return $ Just (file, oldMusic)
+                _ -> do
+                    sfMusic_Stop oldMusic
+                    sfMusic_Destroy oldMusic
+                    startNewMusic
+        Nothing -> startNewMusic
+  where
+    startNewMusic = do
+        music <- sfMusic_CreateFromFile file
+        sfMusic_SetLoop music looped
+        sfMusic_SetVolume music ((fromMaybe 1 volume) * 100)
+        sfMusic_Play music
+        return $ Just (file, music)
 
 -- | Stops any background music that is playing.
--- stopMusic :: IO ()
--- stopMusic = error "stopMusic"
+stopMusic :: IO ()
+stopMusic = modifyMVar_ _globalMusic $ \ mOldMusic -> do
+    forM_ mOldMusic $ \ (_, oldMusic) -> do
+        sfMusic_Stop oldMusic
+        sfMusic_Destroy oldMusic
+    return Nothing
+
+-- | Pauses the current music.
+pauseMusic :: IO ()
+pauseMusic = modifyMVar_ _globalMusic $ \ mMusic -> do
+    mapM_ (sfMusic_Pause . snd) mMusic
+    return mMusic
+
+
+{-# noinline _globalMusic #-}
+-- all music-related functions are synchronized by this global MVar.
+_globalMusic :: MVar (Maybe (FilePath, Ptr Music))
+_globalMusic = unsafePerformIO $ newMVar Nothing
